﻿/*
	This file is part of Warzone Stat Editor by crab
    Copyright © 2011, 2012 Vladimir Salnikov
 
	Warzone Stat Editor by crab is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Warzone Stat Editor by crab is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Warzone Stat Editor by crab; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.IO;


namespace WZStats
{
    public class StatMetadata
    {
        public enum ExType { Unspecified, List, Expression, CodeFunction, LinkTable, ListFromFile };
        public enum FuncType { Unspecified, ResearchPoints, ResearchPriceSum };
        public DataTable data;
        private int col_count;
        public void Load(string filename)
        {
            data = new DataTable();
            DataColumn dc = new DataColumn("filename", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("Header column name", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("DataType", typeof(String)); //int or string
            data.Columns.Add(dc);
            dc = new DataColumn("ExpressionType", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("Expression", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("Caption", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("Description", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("ColumnWidth", typeof(Int32));
            data.Columns.Add(dc);
            dc = new DataColumn("CategoryName", typeof(String));
            data.Columns.Add(dc);
            dc = new DataColumn("ColumnVisibleIndex", typeof(Int32));
            data.Columns.Add(dc);
            dc = new DataColumn("CategoryVisibleIndex", typeof(Int32));
            data.Columns.Add(dc);

            //Expression fields
            //Warning: such fields should be placed at the end of fields list
            col_count = data.Columns.Count;
            dc = new DataColumn("ExType", typeof(ExType));
            data.Columns.Add(dc);
            dc = new DataColumn("IsInteger", typeof(Boolean));
            dc.Expression = "DataType = 'int'";
            data.Columns.Add(dc);
            dc = new DataColumn("CodeFunctionType", typeof(FuncType));
            data.Columns.Add(dc);

            data.ColumnChanged += Column_Changed;

            string[] lines = File.ReadAllLines(filename);
            foreach (string line in lines)
            {
                string[] cells = line.Split(',');
                if (cells.Length != col_count)
                {
                    Exception err = new Exception("Error loading stat files metadata. Filename: "+filename +
                    "\r\nError: invalid number of columns in csv file. Expected number: " + col_count.ToString());
                    throw err;
                }
                DataRow row = data.NewRow();
                for (int i = 0; i < col_count; i++)
                {
                    if(data.Columns[i].DataType == typeof(Int32))
                    {
                        int value = 0;
                        if(Int32.TryParse(cells[i],out value))
                        {
                            row[i] = value;
                        }else
                        {
                            Exception err = new Exception("Error loading stat files metadata. Filename: "+filename +
                            "\r\nError: cannot convert value to Int32. Column: " + i.ToString() + " Row = " + line);
                        }
                    }else
                        row[i] = cells[i];
                }
                data.Rows.Add(row);
            }
        }

        public void Save(string filename)
        {
            string[] lines = new string[data.Rows.Count];
            for (int i = 0; i < data.Rows.Count; i++)
            {
                string[] cells = new string[col_count];
                for (int j = 0; j < col_count; j++)
                {
                    cells[j] = data.Rows[i][j].ToString();
                }
                lines[i] = String.Join(",", cells);
            }
                
            File.WriteAllLines(filename, lines);
        }

        public DataRow[] SelectFields(string filename, string additional_where_str = "")
        {
            if(additional_where_str == "")
                return data.Select("filename = '" + filename + "'");
            else
                return data.Select("filename = '" + filename + "' AND (" + additional_where_str + ")");
        }

        public static string GetExType(ExType extype)
        {
            switch (extype)
            {
                case ExType.Unspecified:
                    return "";
                case ExType.List:
                    return "list";
                case ExType.ListFromFile:
                    return "list from file";
                case ExType.LinkTable:
                    return "link table";
                case ExType.Expression:
                    return "expression";
                case ExType.CodeFunction:
                    return "code function";
            }
            return "";
        }

        public ExType GetExType(string extype)
        {
            switch (extype)
            {
                case "":
                    return ExType.Unspecified;
                case "list":
                    return ExType.List;
                case "list from file":
                    return ExType.ListFromFile;
                case "link table":
                    return ExType.LinkTable;
                case "expression":
                    return ExType.Expression;
                case "code function":
                    return ExType.CodeFunction;
            }
            return ExType.Unspecified;
        }

        public FuncType GetFuncType(string func_type)
        {
            func_type = func_type.ToLower().Replace(" ", "_");
            switch (func_type)
            {
                case "function:research_points":
                    return FuncType.ResearchPoints;
                case "function:research_price_sum":
                    return FuncType.ResearchPriceSum;
            }
            return FuncType.Unspecified;
        }

        private void Column_Changed(object sender, DataColumnChangeEventArgs e )
        {
            if (e.Column.ColumnName == "ExpressionType")
            {
                e.Row["ExType"] = GetExType((string)e.Row["ExpressionType"]);
            }
            if (e.Column.ColumnName == "Expression")
            {
                e.Row["CodeFunctionType"] = GetFuncType((string)e.Row["Expression"]);
            }
        }

    }
}
